/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package at.tfr.aero2.server;

import at.tfr.aero2.server.CdiUtil;
import at.tfr.aero2.server.Logged;
import java.util.Iterator;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import org.jboss.errai.codegen.meta.MetaClass;
import org.jboss.errai.codegen.meta.MetaClassMember;
import org.jboss.errai.codegen.meta.MetaField;
import org.jboss.errai.codegen.meta.MetaMethod;
import org.jboss.errai.codegen.meta.impl.java.JavaReflectionClass;
import org.jboss.errai.marshalling.rebind.DefinitionsFactorySingleton;
import org.jboss.errai.marshalling.rebind.api.impl.defaultjava.DefaultJavaDefinitionMapper;
import org.jboss.errai.marshalling.rebind.api.model.MappingDefinition;
import org.jboss.errai.marshalling.rebind.api.model.MemberMapping;
import org.jboss.errai.marshalling.rebind.api.model.impl.SimpleFactoryMapping;

/**
 *
 * @author Thomas Frühbeck
 */
@Logged
public class EntityMappingDefinition extends MappingDefinition {

   Class clazz;
   MetaClass thisClass;
   MetaClass entityClass;
   MappingDefinition javaDefinition;
   
   public EntityMappingDefinition(Class<?> clazz, boolean doNotGenerate) {
      super(clazz, doNotGenerate);
      this.clazz = clazz;
      CdiUtil.registerMapping(clazz, this);
      thisClass = JavaReflectionClass.newInstance(this.getClass());
      entityClass = JavaReflectionClass.newInstance(clazz);
      
      SimpleFactoryMapping factoryMapping = new SimpleFactoryMapping();
      MetaMethod ctorMethod = thisClass.getMethod("getEntity", Object.class);
      if (ctorMethod == null) {
    	  throw new RuntimeException("EntityMapping for "+entityClass+" must implement static method getEntity(Object)!");
      }
      factoryMapping.setMethod(ctorMethod);
      factoryMapping.mapParmToIndex("id", 0, Long.class);
      setInstantiationMapping(factoryMapping);
      
   }
   
   public void updateEntityMarshaller() {
      javaDefinition = DefaultJavaDefinitionMapper.map(entityClass, DefinitionsFactorySingleton.get());
      setMarshallerInstance(new EntityMarshaller(entityClass, this));
      setClientMarshallerClass(javaDefinition.getClientMarshallerClass());

      for (MetaField field : entityClass.getDeclaredFields()) {

         if (field.isAnnotationPresent(ManyToOne.class) || 
                 field.isAnnotationPresent(OneToOne.class) ||
                 field.isAnnotationPresent(OneToMany.class)) {
            Iterator<MemberMapping> members = javaDefinition.getMemberMappings().iterator();
            while (members.hasNext()) {
               MemberMapping member = members.next();
               if (member.getKey().equals(field.getName())) {
                  members.remove();
                  break;
               }
            }
         }
      }
   }

   /**
    * @see DefaultJavaDefinitionMapper#map(org.jboss.errai.codegen.meta.MetaClass, org.jboss.errai.marshalling.rebind.DefinitionsFactory)
    * @param field
    * @return
    */
   MemberMapping getMapping(final MetaField field, final MetaClass toMap, final MetaClass targetClass) {
      final String toMapName = toMap.getFullyQualifiedName();
      MemberMapping mapping = new MemberMapping() {
         private MetaClass type = (field.getType().isArray() ? field.getType() : field.getType());
         private final MetaClass targetType = targetClass;

         @Override
         public MetaClassMember getBindingMember() {
            return field;
         }

         @Override
         public MetaClassMember getReadingMember() {
            return field;
         }

         @Override
         public String getKey() {
            return field.getName();
         }

         @Override
         public MetaClass getType() {
            return type;
         }

         @Override
         public void setType(final MetaClass type) {
            this.type = type;
         }

         @Override
         public MetaClass getTargetType() {
            return targetType;
         }

         @Override
         public boolean canRead() {
            return true;
         }

         @Override
         public boolean canWrite() {
            return true;
         }

         @Override
         public void setMappingClass(final MetaClass clazz) {
         }

         @Override
         public String toString() {
            return toMapName + "#" + field.getName();
         }
      };
      return mapping;
   }

   MetaField getKey(MetaClass entity) {
      for (MetaField field : entity.getDeclaredFields()) {
         if (field.isAnnotationPresent(Id.class)) {
            return field;
         }
      }
      return null;
   }

   public MappingDefinition getJavaDefinition() {
      return javaDefinition;
   }
   
}
